// markov.cpp
#include <cmath>
#include "Markov.h"

void Markov::setInitialValues(vector<double>& y){
    y[0] = 1.0;
    for(int i = 1 ; i < NUM_OF_NA_STATES ; i++){
        y[i] = 0.0;
    }
    vold = -1000.0;
};
void Markov::rhs(double t, vector<double> y, vector<double>& dy){
    double v = t<20.0 || t>= 30.0 ? -100.0 : 0.0;
    if(v != vold){
        a11 = 3.802/(0.1027*exp(-v/17.0) + 0.20*exp(-v/150));
        a12 = 3.802/(0.1027*exp(-v/15.0) + 0.23*exp(-v/150));
        a13 = 3.802/(0.1027*exp(-v/12.0) + 0.25*exp(-v/150));

        b11 = 0.1917*exp(-v/20.3);
        b12 = 0.20*exp(-(v-5)/20.3);
        b13 = 0.22*exp(-(v-10)/20.3);

        a3 = (3.7933e-7)*exp(-v/7.7);
        b3 = 0.0084 + 0.00002*v;

        a2 = 9.178*exp(v/29.68);
        b2 = (a13*a2*a3)/(b13*b3);

        a4 = a2/100;
        b4 = a3;

        a5 = a2/(9.5e4);
        b5 = a3/50;
        vold = v;
    }
    dy[0] = -(a11 + b3)*y[0] + b11*y[1] + a3*y[4];
    dy[1] = -(b11 + b3 + a12)*y[1] + a11*y[0] + b12*y[2] + a3*y[5];
    dy[2] = -(b12 + b3 + a13)*y[2] + a12*y[1] + b13*y[3] + a3*y[6];
    dy[3] = -(b13 + a2)*y[3] + a13*y[2] + b2*y[6];
    dy[4] = -(a11 + a3)*y[4] + b11*y[5] + b3*y[0];
    dy[5] = -(b11 + a3 + a12)*y[5] + a11*y[4] + b12*y[6] + b3*y[1];
    dy[6] = -(b12 + a3 + b2 + a4)*y[6] + a12*y[5] + b3*y[2] + a2*y[3] + b4*y[7];
    dy[7] = -(b4 + a5)*y[7] + a4*y[6] + b5*y[8];
    dy[8] = -b5*y[8] + a5*y[7];
}
