// main.cpp of channel
#include <iostream>
#include <fstream>
#include <cmath>
#include <algorithm>

#include "Markov.h"

int main(){
    const int ndim = NUM_OF_NA_STATES;
    const double dt = 0.01;
    const double totalTime = 20.0;
    const int npts = (int)(totalTime/dt)+1;
    const int ntraces = 14;
    const double ENa = 50.0;

    std::vector<double> y(ndim);
    std::vector<double> vps(ntraces);
    for(int n=0;n<ntraces;n++){
        vps[n] = -60.0 + 10.0*n;
    }

    Markov ce;
    ce.initRK4(ndim, dt);

    std::vector<std::vector<double> > tr(ntraces, std::vector<double>(npts));
    for(int n=0;n<ntraces;n++){
       ce.setInitialValues(y);
        double vp = -60.0 + 10.0*n;
        ce.setPulse(vp);
        for(int i=0; i<npts; i++){
            double t = dt*i;
            tr[n][i] = (ce.getVoltage(t)-ENa)*y[3];
            ce.advanceStep(t, y);
        }
    }
    std::fstream fd("ax.txt", std::ios::out | std::ios::trunc);
    for(int i = 0;i<npts;i++){
        fd << dt*i;
        for(int n=0;n<ntraces;n++){
            fd << " " << tr[n][i];
        }
        fd << std::endl;
    }
    fd.close();


    std::fstream fd2("bx.txt", std::ios::out | std::ios::trunc);
    for(int n=0;n<ntraces;n++){
        double pmin, pmax;
        pmin = pmax = tr[n][0];
        for(int i=1;i<npts;i++){
            double p = tr[n][i];
            if(tr[n][i]<pmin){
                pmin = p;
            }else if(p > pmax){
                pmax = p;
            }
        }
        fd2 << -60.0+10.0*n;
        if(-pmin>=pmax){
            fd2 << " " << pmin << std::endl;
        }else{
            fd2 << " " << pmax << std::endl;
        }
    }
    fd2.close();

#if defined(_WIN32)
    system("..\\..\\tools\\gc ax.txt &");
    system("..\\..\\tools\\gc bx.txt &");
    #else
    system("../../tools/gc ax.txt &");
    system("../../tools/gc bx.txt &");
#endif
    return 0;
}
