//  cell.cpp
#include<iostream>
#include <cmath>
#include "Cell.h"

void Cell::setInitialValues(double v_init, std::vector<double>& y){
    Am = 3.14 * Dm * Lm;    // (squre_micrometer) surface area
    Cm = 0.01 * Am;
    InvCm = 1.0/Cm;

    Gleak = Am * dGleak * fac;
    GNa   = Am * dGNa * fac;
    GK    = Am * dGK * fac;
    GTCa  = Am * dGTCa * fac;

    double v = y[0] = v_init;
    double a, b, tau, inf;
    get_Na_m(v, &a, &b);
    y[1] = a/(a+b);

    get_Na_h(v, &a, &b);
    y[2] = a/(a+b);

    get_K_n(v, &a, &b);
    y[3] = a/(a+b);

    get_TCa_m(v, &tau, &inf);
    y[4] = inf;

    get_TCa_h(v, &tau, &inf);
    y[5] = inf;
}
void Cell::setPulse(double ton, double toff, double current){
    Ton  = ton;
    Toff = toff;
    Iinj = current;
}
void Cell::get_Na_m(double v, double* a, double* b){
    double v2 = v-vtraub;
    *a = 0.32*(13.0-v2)/(exp((13.0-v2)/4.0)-1.0);
    *b = 0.28*(v2-40.0)/(exp((v2-40.0)/5.0)-1.0);
}
void Cell::get_Na_h(double v, double* a, double* b){
    double v2 = v-vtraub;
    *a = 0.128 * exp((17.0-v2)/18.0);
    *b = 4.0/(1.0 + exp((40.0-v2)/5.0));
}
void Cell::get_K_n(double v, double* a, double* b){
    double v2 = v - vtraub;
    *a = 0.032*(15.0-v2)/(exp((15.0-v2)/5.0)-1.0);
    *b = 0.5*exp((10.0-v2)/40.0);
}
void Cell::get_TCa_m(double v, double* tau, double* inf){
    double vshift=0.0;
    double vs = v + vshift;
    *tau = (3.0+1.0/(exp((vs+25.0)/10.0)+exp(-(vs+100.0)/15.0)));
    *inf = 1.0/(1.0+exp(-(vs+50.0)/7.4));
}
void Cell::get_TCa_h(double v, double* tau, double* inf){
    double vshift=0.0;
    double vs = v + vshift;
    *tau = (85.0+1.0/(exp((vs+46.0)/4.0)+exp(-(vs+405.0)/50.0))) ;
    *inf = 1.0/(1.0+exp((vs+78.0)/5.0));
}

void Cell::rhs(double tt, std::vector<double> y, std::vector<double>& dy){
    double INa, IK, ITCa;
    double a, b, tau, inf;
    double shift = 0.0;
    double v = y[0];
    double c = (tt<Ton || tt>Toff)? 0.0 : Iinj;

    INa = GNa * pow(y[1],3) * y[2]  * (v-ENa); // INa = GNa * m^3 * h * (v-Ena)
    IK = GK * pow(y[3],4) * (v-EK);            // IK = GK * n^4 (v-EK)
    ITCa = GTCa * pow(y[4],2) * y[5] * (v - ECa);
    dy[0] = InvCm * (c - Gleak * (v-Eleak) - INa - IK - ITCa);
    // Na channel
    get_Na_m(v, &a, &b);
    dy[1] = a - (a+b)*y[1];
    get_Na_h(v, &a, &b);
    dy[2] = a - (a+b)*y[2];
    // K channel (delayed rectifier)
    get_K_n(v, &a, &b);
    dy[3] = a - (a+b)*y[3];
    // T-type Ca channel
    get_TCa_m(v, &tau, &inf);
    dy[4] = (inf - y[4]) / tau;
    get_TCa_h(v, &tau, &inf);
    dy[5] = (inf - y[5]) / tau;
}
