// main.cpp of cellpot
#include <iostream>
#include <fstream>
#include "Cell.h"

int main(){
    const int m = 6;                // number of equations
    const double dt = 0.05;         // step size
    const double totalTime = 600.0; // simulation time
    const int npts = (int)(totalTime/dt)+1;
    vector<double> y(m);
    vector<double> vol(npts);

    double vinit = -63.27;  // initial voltate
    double Iinj  = -2000.0; // current injection(pA)
    double Ton   = 50.0;    // on-time
    double Toff  = 300.0;   // off-time

    Cell ce;
    ce.initRK4(m, dt);
    ce.setInitialValues(vinit, y);
    ce.setPulse(Ton, Toff, Iinj);

    std::fstream fd("ax.txt", std::ios::out | std::ios::trunc);
    for(int i=0; i<npts; i++){
        double t = dt*i;
        vol[i] = y[0];
        fd << t << " " << y[0] << std::endl;
        ce.advanceStep(t, y);
    }
    // count the number of spikes
    int sn=0;
    double th = -20.0;
    for(int i=0;i<npts-1;i++){
        if(vol[i+1]>=th && vol[i] < th) sn++;
    }
    std::cout << "number of spikes = " << sn << std::endl;
    fd.close();
#if defined(_WIN32)
    system("..\\..\\tools\\gc ax.txt &");
#else
    system("../../tools/gc ax.txt &");
#endif
    return 0;
}
