% release.m
% Stochastic neurotransmitter release: ex12.m
% Ralf Schneggenburger & Erwin Neher
% Nature 406, 889-893, 2000}

function main()
    tic
    
    ttime = 4.0;    % total time
    dt    = 0.02;   % time step
    peak  = 20.0;   % peak calcium concentration
    hw    = 1.0;    % half width of calcium transient
    iter  = 100000;  % number of trials

    npts = int32(ttime/dt)+1;
    tt  = linspace(0.0, ttime, npts);
    sum = zeros(1, npts, 'int32');
    ca  = zeros(1, npts, 'double');

    sprintf('%d iterations\n', iter)
    
    % ca transient
    d = 2.0*sqrt((1.0 - sqrt(0.5)))/hw;
    k = int32(2.0/(dt*d));
    ca(1:k) = peak * (((d*tt(1:k) - 1.0).^2.0)-1.0).^2 ;

    % kinetic constants
    kon    = 9.0e-2; % microM^{-1} ms^{-1}
    koff   = 9.5;    % ms^{-1}
    beta   = 0.25;   % cooperativity
    gamma  = 6.0;    % 6.0d0  ! ms^{-1}
    epsilon= 0.3;    % ms^{-1}
    initialState = 1;

    konh   = dt * kon;
    koffh  = dt * koff;
    gammah = dt * gamma;
    epsilonh = dt*epsilon;
    beta2  = beta*beta;
    beta3  = beta*beta2;
    beta4  = beta*beta3;

    counter = 0;
    for k = 1:iter
        s  = initialState;  %initial state
        for i = 1:npts
            r = rand;              % get a random number [0, 1]
            alpha = konh * ca(i);  % get Ca level
            snext = s;
            switch s
                case 1
                    th = 5.0 * alpha;
                    if r<th
                        snext = 2;
                    end
                case 2
                    th = 4.0 * alpha;
                    if r < th
                        snext = 3;
                    else
                        th = th + koffh;
                        if r<th
                            snext = 1;
                        end
                    end
                case 3
                    th = 3.0 * alpha;
                    if r < th
                        snext = 4;
                    else
                        th = th + koffh * beta;
                        if r < th
                            snext = 2;
                        end
                    end
                case 4
                    th = 2.0 * alpha;
                    if r < th
                        snext = 5;
                    else
                        th = th + koffh * beta2;
                        if r < th
                            snext = 3;
                        end
                    end
                case  5
                    th = alpha;
                    if r < th
                        snext = 6;
                    else
                        th = th + koffh * beta3;
                        if r < th
                            snext = 4;
                        end
                    end
                case 6
                    th = gammah;
                    if r < th
                        snext = 7;
                    else
                        th = th + koffh * beta4;
                        if r < th
                            snext = 5;
                        end
                    end
                case  7      % release
                    snext = 0;           
                    sum(i) = sum(i) + 1;
                    counter = counter + 1;
                case  0      %recovery from release
                    th = epsilonh;
                    if r < th
                        snext = 1;
                    end
            end % end of switch       
            s = snext;
        end  % end of i-loop
    end % end of k-loop
    sprintf('release probability = %f\n', counter/iter)    
    toc
    plot(tt,sum, tt, ca)    
end
