# channel.py
import numpy as np
import math
from scipy.integrate import odeint
import matplotlib.pyplot as plt

def channel():
    ndim = 9
    dt = 0.01
    totalTime = 20.0;
    npts = (int)(totalTime/dt)+1;
    ntraces = 16
    vold = -1000.0
    a2 = a3 = a4 = a5 = a11 = a12 = a13 = 0.0
    b2 = b3 = b4 = b5 = b11 = b12 = b13 = 0.0
    ENa = 50.0
    vpulse = 0.0;

    def getVoltage(t):
        if t>=5.0 and t<15.0:
            vp = vpulse
        else:
            vp = -100.0
        return vp

    def rhs(y, t):
        v =  getVoltage(t)
        if v != vold:
            a11 = 3.802/(0.1027*math.exp(-v/17.0) + 0.20*math.exp(-v/150.0))
            a12 = 3.802/(0.1027*math.exp(-v/15.0) + 0.23*math.exp(-v/150.0))
            a13 = 3.802/(0.1027*math.exp(-v/12.0) + 0.25*math.exp(-v/150.0))

            b11 = 0.1917*math.exp(-v/20.3)
            b12 = 0.20*math.exp(-(v-5.0)/20.3)
            b13 = 0.22*math.exp(-(v-10.0)/20.3)

            a3 = (3.7933e-7)*math.exp(-v/7.7)
            b3 = 0.0084 + 0.00002*v

            a2 = 9.178*math.exp(v/29.68)
            b2 = (a13*a2*a3)/(b13*b3)

            a4 = a2/100.0
            b4 = a3

            a5 = a2/(9.5e4)
            b5 = a3/50.0
            v = vold
        return np.array(
            [-(a11 + b3)*y[0] + b11*y[1] + a3*y[4],
             -(b11 + b3 + a12)*y[1] + a11*y[0] + b12*y[2] + a3*y[5],
             -(b12 + b3 + a13)*y[2] + a12*y[1] + b13*y[3] + a3*y[6],
             -(b13 + a2)*y[3] + a13*y[2] + b2*y[6],
             -(a11 + a3)*y[4] + b11*y[5] + b3*y[0],
             -(b11 + a3 + a12)*y[5] + a11*y[4] + b12*y[6] + b3*y[1],
             -(b12 + a3 + b2 + a4)*y[6] + a12*y[5] + b3*y[2] + a2*y[3] + b4*y[7],
             -(b4 + a5)*y[7] + a4*y[6] + b5*y[8],
             -b5*y[8] + a5*y[7] ])

    tr = np.zeros((ntraces,npts),float)
    t = np.linspace(0.0, totalTime, npts)
    vps = np.linspace(-60.0, -60.0+10.0*(ntraces-1), ntraces)
    vv = np.zeros(npts)
    for n in range(ntraces):
        y0 = np.zeros(ndim)
        y0[0] = 1.0
        vpulse = vps[n]
        for i in range(npts):
            vv[i] = getVoltage(dt*i)
        y = odeint(rhs, y0, t)
        tr[n,:] = y[:,3]*(vv - ENa)

    peak=np.zeros(ntraces)
    for n in range(ntraces):
        pmin = min(tr[n,:])
        pmax = max(tr[n,:])
        if(-pmin>=pmax):
            peak[n] = pmin
        else:
            peak[n] = pmax

    plt.subplot(2,1,1)
    for n in range(ntraces):
        plt.plot(t, tr[n,:])
    plt.xlabel('time (ms)')
    plt.ylabel('current amplitude')
    plt.subplot(2,1,2)
    plt.plot(vps, peak, 'o')
    plt.xlabel('pulse voltage (mV)')
    plt.ylabel('current amplitude')
    plt.show()

if __name__ == '__main__':
    channel()
